;;; custom-load.el --- automatically extracted custom dependencies

;;; Code:

(autoload 'custom-add-loads "cus-load")

(custom-add-loads 'lookup-faces '("lookup-vars"))
(custom-add-loads 'lookup-general-options '("lookup-vars"))
(custom-add-loads 'ndspell '("ndspell"))
(custom-add-loads 'applications '("lookup-vars"))
(custom-add-loads 'lookup-setup-variables '("lookup-vars"))
(custom-add-loads 'lookup-agents '("ndeb" "ndic" "ndkks" "ndnmz" "ndspell" "ndsrd"))
(custom-add-loads 'lookup-setup '("lookup-package"))
(custom-add-loads 'lookup '("lookup-vars"))
(custom-add-loads 'ndsrd '("ndsrd"))
(custom-add-loads 'ndic '("ndic"))
(custom-add-loads 'ndeb '("ndeb"))
(custom-add-loads 'ndnmz '("ndnmz"))
(custom-add-loads 'ndkks '("ndkks"))

;;; custom-load.el ends here
