;;; DO NOT MODIFY THIS FILE
(if (featurep 'lookup-autoloads) (error "Already loaded"))

;;;### (autoloads nil "_pkg" "lisp/_pkg.el")

(package-provide 'lookup :version 1.15 :author-version "1.0" :type 'regular)

;;;***

;;;### (autoloads (lookup-entry-display) "lookup-entry" "lisp/lookup-entry.el")

(autoload 'lookup-entry-display "lookup-entry" nil nil nil)

;;;***

;;;### (autoloads (lookup-use-package) "lookup-package" "lisp/lookup-package.el")

(autoload 'lookup-use-package "lookup-package" "\
補助パッケージをロードする。
AGENT-ID は、パッケージを適用するエージェントID。
PACKAGE-NAME は、パッケージ名、あるいはパッケージの絶対パス。
絶対パスでない場合、パッケージは変数 `lookup-package-directory' が
示すディレクトリに展開しておく必要がある。
この関数は `lookup-agent-options-alist' 及び
`lookup-dictionary-options-alist' の値を書き換える。" nil nil)

;;;***

;;;### (autoloads (lookup-select-display) "lookup-select" "lisp/lookup-select.el")

(autoload 'lookup-select-display "lookup-select" nil nil nil)

;;;***

;;;### (autoloads (lookup-vse-get-menu lookup-vse-search-query-internal lookup-vse-search-query) "lookup-vse" "lisp/lookup-vse.el")

(autoload 'lookup-vse-search-query "lookup-vse" nil nil nil)

(autoload 'lookup-vse-search-query-internal "lookup-vse" nil nil nil)

(autoload 'lookup-vse-get-menu "lookup-vse" nil nil nil)

;;;***

;;;### (autoloads (lookup-module-backward lookup-module-forward lookup-secondary-other-frame lookup-secondary-full-screen lookup-secondary lookup-selection-other-frame lookup-selection-full-screen lookup-selection lookup-region-other-frame lookup-region-full-screen lookup-region lookup-word-other-frame lookup-word-full-screen lookup-word lookup-pattern-other-frame lookup-pattern-full-screen lookup-pattern lookup-setup lookup lookup-version) "lookup" "lisp/lookup.el")

(autoload 'lookup-version "lookup" "\
Display the version string of Lookup.
With prefix argument, insert string at point." t nil)

(autoload 'lookup "lookup" "\
Lookup を起動し、設定した辞書のリストを表示する。
既に起動している場合は、最後に検索したときの状態を取り戻す。" t nil)

(autoload 'lookup-setup "lookup" "\
Lookup の全てのセットアップを行なう。
初期化ファイルの読み込み、エージェントの立ち上げ、辞書の初期化等を実行する。
それ以外は何もしない。" t nil)

(autoload 'lookup-pattern "lookup" "\
Search for the PATTERN." t nil)

(autoload 'lookup-pattern-full-screen "lookup" "\
Search for the PATTERN in full screen." t nil)

(autoload 'lookup-pattern-other-frame "lookup" "\
Search for the PATTERN in another frame." t nil)

(autoload 'lookup-word "lookup" "\
Search for the word near the cursor." t nil)

(autoload 'lookup-word-full-screen "lookup" "\
Search for the word near the cursor in full screen." t nil)

(autoload 'lookup-word-other-frame "lookup" "\
Search for the word near the cursor in another frame." t nil)

(autoload 'lookup-region "lookup" "\
Search for the region." t nil)

(autoload 'lookup-region-full-screen "lookup" "\
Search for the region in full screen." t nil)

(autoload 'lookup-region-other-frame "lookup" "\
Search for the region in another frame." t nil)

(autoload 'lookup-selection "lookup" "\
Search for the mouse's selection." t nil)

(autoload 'lookup-selection-full-screen "lookup" "\
Search for the mouse's selection in full screen." t nil)

(autoload 'lookup-selection-other-frame "lookup" "\
Search for the mouse's selection in another frame." t nil)

(autoload 'lookup-secondary "lookup" "\
Search for the mouse's secondary selection." t nil)

(autoload 'lookup-secondary-full-screen "lookup" "\
Search for the mouse's secondary selection in full screen." t nil)

(autoload 'lookup-secondary-other-frame "lookup" "\
Search for the mouse's secondary selection in another frame." t nil)

(autoload 'lookup-module-forward "lookup" "\
モジュールを前に切り替える。" t nil)

(autoload 'lookup-module-backward "lookup" "\
モジュールを後に切り替える。" t nil)

;;;***

;;;### (autoloads (lookup-make-url-reference) "ndmisc" "lisp/ndmisc.el")

(autoload 'lookup-make-url-reference "ndmisc" nil nil nil)

;;;***

;;;### (autoloads (stem-english) "stem-english" "lisp/stem-english.el")

(autoload 'stem-english "stem-english" "\
活用語尾を取り除く関数
与えられた語の元の語として可能性のある語の文字列長の昇順のリストを返す" nil nil)

;;;***

(provide 'lookup-autoloads)
